// Copyright (c) 2013 Jasper den Ouden, under the MIT license, 
// see ../doc/mit.txt from the project directory.
#version 3.7;//.0.RC4-2(pacman) //Created with this version.

#include "colors.inc"
#include "functions.inc"

// Top part; upto the randoms is from
//  http://springrts.com/wiki/Create_map_texture_with_povray

// Setup some parameters that we will use within the script
#declare MAP_WIDTH= 2*pi;
#declare MAP_HEIGHT= 2;
#declare MAP_EL=600; // elevation

global_settings { 
  // ambient light level
  ambient_light rgb<1,1,1> 
}

// Set a directional light
light_source {
  <100,-100,100> * 20, color rgb 1
  parallel
  point_at <1, 0, 0>
}

// Set the background colour
background {rgb<1,1,1,1>}

// Define our camera position, and set it to "orthographic" (important :)
camera {
  orthographic
  location <0,0,MAP_EL + 90>    // position & direction of view
  look_at  <0,0,0>
  right MAP_WIDTH*x            // horizontal size of view
  up MAP_HEIGHT*z               // vertical size of view
}

#declare cloud_rnd_x = 10265;
#declare cloud_rnd_y = 3669;
#declare cloud_rnd_z = 32390;
#declare agate_rnd_x = 32364;
#declare agate_rnd_y = 1342;
#declare agate_rnd_z = 15055;

#declare cloud_angle_rnd_x = 14452;
#declare cloud_angle_rnd_y = 432;
#declare cloud_angle_rnd_z = 1759;

#declare cloud_h = 0.3*0.0249;
#declare air_density = 4;

#declare rotate_angle = 2*clock*180/pi;

#declare surf_dist = function(x,y,z)
{ sqrt(x*x+y*y+z*z)-1 }

#declare air = media {
  absorption rgb<0,0.5,0.6> //Lower these
  scattering{1, rgb<0.3,0.3,0.6>}
  density {
    function{ air_density*exp(-surf_dist(x,y,z)/(0.3*0.0249))}
    color_map {
      [0.0 color rgb <0,0,0>]
      [1.0 color rgb <0.6,0.6,1>] } }} //less R

#declare agate_tmplt = function
  { pigment{ agate
             color_map { [0.0 color rgb <0, 0, 0>]
	                 [1.0 color rgb <1, 1, 1>] }
    translate <agate_rnd_x,agate_rnd_y,agate_rnd_z>
  } }

#declare agate_fun = function(x,y,z)
  { agate_tmplt(x,y,z).red }
  
#declare angle_time = function(x,y,z)
  { f_noise3d(2*x + cloud_angle_rnd_x,
              2*(y+ 2*cos(clock)) + cloud_angle_rnd_y,
              2*(z+ 2*sin(clock))  + cloud_angle_rnd_z) }
  
//This 'flows' planet-scale i want 'dynamics'
#declare cloud_comb = function(x,y,z) 
  { agate_fun(x, y + 2*sin(angle_time(x,y,z)), z + 2*cos(angle_time(x,y,z)))
    *f_noise3d(x,y +sin(angle_time(x,y,z)),z +cos(angle_time(x,y,z)))
  }
  
//Thanks to http://geomorph.sourceforge.net
#declare CLOUDS = 
  pigment {
    function{ cloud_comb(x,y,z) }
    translate <cloud_rnd_x,cloud_rnd_y,cloud_rnd_z>
    warp{ turbulence 0.2+0.05*cos(clock) }
    color_map {
      [0.2 - 0.2*0.67 rgbt <1,1,1,1>] 
      [0.95- 0.2*0.67 rgbt <0.9,0.9,0.9,0.1>] 
      [1.0                  rgbt <0.8,0.8,0.8,0.2>]
    }
  }
  
#declare SMALLER_CLOUDS =
  pigment {
    function{ cloud_comb(x,y,z) }
    translate <cloud_rnd_x,cloud_rnd_y,cloud_rnd_z>
    warp{ turbulence 0.2+0.05*cos(clock) }
    color_map {
      [0.4 - 0.2*0.67 rgbt <1,1,1,1>] 
      [0.95- 0.2*0.67 rgbt <1,1,1,0.2>]
      [1.0                  rgbt <0.9,0.9,0.9,0.2>]
    }
  }
