// Copyright (c) 2013 Jasper den Ouden, under the MIT license, 
// see ../doc/mit.txt from the project directory.
#version 3.7;//.0.RC4-2(pacman) //Created with this version.

#include "colors.inc"
#include "functions.inc"

// Top part; upto the randoms is from
//  http://springrts.com/wiki/Create_map_texture_with_povray

// Setup some parameters that we will use within the script
#declare MAP_WIDTH= 2*pi;
#declare MAP_HEIGHT= 2;
#declare MAP_EL=600; // elevation

global_settings { 
  // ambient light level
  ambient_light rgb<1,1,1> 
}

// Set a directional light
light_source {
  <100,-100,100> * 20, color rgb 1
  parallel
  point_at <1, 0, 0>
}

// Set the background colour
background {rgb<1,1,1>}

// Define our camera position, and set it to "orthographic" (important :)
camera {
  orthographic
  location <0,0,MAP_EL + 90>    // position & direction of view
  look_at  <0,0,0>
  right MAP_WIDTH*x            // horizontal size of view
  up MAP_HEIGHT*z               // vertical size of view
}

#declare wrink_rnd_x = 29176;
#declare wrink_rnd_y = 21429;
#declare wrink_rnd_z = 9692;

#declare base_rnd_x = 7402;
#declare base_rnd_y = 29703;
#declare base_rnd_z = 5325;

#declare ice_t = 0.6; //The below not changable as-is.
#declare best_temp = 0.8;
#declare temp_range = 0.2;

#declare water_eff_h = 0.03;

#declare water_g = 0;
#declare water_b = 1;

#declare rotate_angle = 2*clock*180/pi;

//Might attempt adding  mountains?
#declare base_hfun = function (x,y,z)
{ f_noise3d(x+base_rnd_x,y+base_rnd_y,z+base_rnd_z) } 

#declare wrinkles_tmplt= function{ pigment{ wrinkles 
  scale 1 
  translate <wrink_rnd_x,wrink_rnd_y,wrink_rnd_z>} }
#declare wrinkles_fun = function(x,y,z)
{ wrinkles_tmplt(x,y,z).red }

#declare hfun = function (x,y,z) {
  max((base_hfun(x,y,z) + 10*(base_hfun(x,y,z)-0.0)*wrinkles_fun(x,y,z))/
      (1+10*(1-0.0)),0) }

#declare light_inpr = function (y){ 1.4*sqrt(1-pow(y*0.211,2)) }

#declare rock_bottom = texture //Rock bottom
  { pigment
    { function{ hfun(x,y,z) }
      color_map 
      { [0   color rgb <.6, .4, .1>] //Sand
	[0.4 color rgb <.55, .25, .05>]
	[0.8 color rgb <0.4,0.4,0.4>] //Rock 
	[1   color rgb <0.4,0.4,0.4>] //Rock 
      }
    }
    scale 0.211
    finish
    { diffuse 0.7
      specular 0.0
      roughness 1.0
    }
  }

//Lower regions have more moderate temperatures, weighted average with 
// the north and south.
#declare temperature = function (y,h)
{ (light_inpr(y) + 2*(light_inpr(abs(y)-0.1) + light_inpr(min(abs(y)+0.2,1)))
                    /(1+pow((h-0.0)/0.1,2)))/
  (1+ 4/(1+pow((h-0.0)/0.1,2)))*(1-pow(h,5)*0.03/0.0249)
}

#declare growth_level = function(temp,h)
{ max(0, 1-2*pow(h-0.0,2)*0.03/0.0249)/
  (1 + pow((temp - best_temp)/temp_range,2)) }

#declare growth = texture
  { pigment
    { function
      { min(max(0,growth_level(temperature(y,hfun(x,y,z)), hfun(x,y,z))),1) }
      color_map
      { [0.0,0.5 color rgbft <0,0,0,0,1> color rgbft <.07, .4, .3, 0,1>]
	[0.7 color rgb <.07, .4, .3>]
	[1 color rgb <.07, .4, .3>]
//	[0.95 1 color rgb <.1,0.4,0.2> color rgb <1,0.4,0.3>]
      }
    }
    scale 0.211
    finish
    { diffuse 0.7
      specular 0.0
      roughness 1.0
    }
  }
#declare tropic_level = function(temp,h)
{ max(0, 1-4*pow(h-0.0,2)*0.03/0.0249)/
  (1 + 2*pow((temp - 1.1*best_temp)/temp_range,2)) }

#declare tropic = texture
  { pigment
    { function
      { min(max(0,tropic_level(temperature(y,hfun(x,y,z)), hfun(x,y,z))),1) }
      color_map
      { [0.0,0.5 color rgbft <0,0,0,0,1> color rgbft <.0,.3,0, 0,1>]
	[0.95 color rgb <.0, .3, 0>]
	[1 color rgb <.0, .3, 0>]
      }
    }
    scale 0.211
    finish
    { diffuse 0.7
      specular 0.0
      roughness 1.0
    }
  }

#declare ocean_area = texture { 
  pigment {
    function{ hfun(x,y,z) }
    color_map {
      [0.00, 0.0 color rgb <0,water_g,water_b> //Water
	             color rgb <0,water_g,water_b>]
      [0.0+0.05, 1.0 color rgbft <0,water_g,water_b, 0,1> //No water
                         color rgbft <0,water_g,water_b, 0,1>]
    }
  }
  scale 0.211
  finish
  {
    diffuse 0.7
    specular 0.0
    roughness 0.6
  }  
}

#declare ice_area = texture
  { pigment
    { function{ min(max(0,temperature(y,hfun(x,y,z))),1) }
      color_map
      { [0.0 ice_t color rgb <0.9,0.9,1> color rgb <0.9,0.9,1>]
	[ice_t 1 color rgbft <0.9,0.9,1,0,1> color rgbft <0.9,0.9,1,0,1>]
      }
    }
    scale 0.211
    finish
    { diffuse 0.7
      specular 0.0
      roughness 1.0
    }
  }

plane { <0, 0, 1>, -1
      texture{ rock_bottom }
      texture{ growth }
      texture{ tropic }
      texture{ ocean_area }
      texture{ ice_area }
}