// Copyright (c) 2013 Jasper den Ouden, under the MIT license, 
// see ../doc/mit.txt from the project directory.
#version 3.7;//.0.RC4-2(pacman) //Created with this version.

#include "colors.inc"
#include "functions.inc"

// Top part; upto the randoms is from
//  http://springrts.com/wiki/Create_map_texture_with_povray

// Setup some parameters that we will use within the script
#declare MAP_WIDTH= 2*pi;
#declare MAP_HEIGHT= 2;
#declare MAP_EL=600; // elevation

global_settings { 
  // ambient light level
  ambient_light rgb<1,1,1> 
}

// Set a directional light
light_source {
  <100,-100,100> * 20, color rgb 1
  parallel
  point_at <1, 0, 0>
}

// Set the background colour
background {rgb<1,1,1>}

// Define our camera position, and set it to "orthographic" (important :)
camera {
  orthographic
  location <0,0,MAP_EL + 90>    // position & direction of view
  look_at  <0,0,0>
  right MAP_WIDTH*x            // horizontal size of view
  up MAP_HEIGHT*z               // vertical size of view
}

#declare wrink_rnd_x = RANDOM_4;
#declare wrink_rnd_y = RANDOM_5;
#declare wrink_rnd_z = RANDOM_6;

#declare base_rnd_x = RANDOM_10;
#declare base_rnd_y = RANDOM_11;
#declare base_rnd_z = RANDOM_12;

#declare ice_t = 0.6; //The below not changable as-is.
#declare best_temp = 0.8;

#declare light_inpr = function (y){ SUN_HEAT*sqrt(1-pow(y*PLANET_SCALE,2)) }

#declare temperature = function (y,h)
{ (light_inpr(y) + 2*(light_inpr(abs(y)-0.1) + light_inpr(min(abs(y)+0.2,1)))
                    /(1+pow((h-WATER_H)/0.1,2)))/
  (1+ 4/(1+pow((h-WATER_H)/0.1,2)))*(1-pow(h,5)*DEFAULT_ATM_H/ATM_H)
}

//Might attempt adding  mountains?
#declare base_hfun = function (x,y,z)
{ f_noise3d(x+base_rnd_x,y+base_rnd_y,z+base_rnd_z) } 

#declare wrinkles_tmplt= function{ pigment{ wrinkles 
  scale 1 
  translate <wrink_rnd_x,wrink_rnd_y,wrink_rnd_z>} }
#declare wrinkles_fun = function(x,y,z)
{ wrinkles_tmplt(x,y,z).red }

#declare hfun = function (x,y,z) {
  max((base_hfun(x,y,z) + 10*(base_hfun(x,y,z)-WATER_H)*wrinkles_fun(x,y,z))/
      (1+10*(1-WATER_H)),0) }

#declare normal_fun = function(y,h) { 
  max(WATER_H,h) + min(max(0,10*(ice_t - temperature(y,h))), 0.05)  }

#declare real_hmap = texture
  { pigment
    { function{ normal_fun(y, hfun(x,y,z)) }
      color_map 
      { [0   color rgb <0,0,0>]
	[1   color rgb <1,1,1>]
      }
    }
    scale PLANET_SCALE
  }

plane { <0, 0, 1>, -1
      texture{ real_hmap }
}