// Copyright (c) 2013 Jasper den Ouden, under the MIT license, 
// see ../doc/mit.txt from the project directory.
#version 3.7;//.0.RC4-2(pacman) //Created with this version.

#include "colors.inc"
#include "functions.inc"

global_settings
{ assumed_gamma 1.0
  radiosity {
    pretrace_start 0.08
    pretrace_end   0.01
    count 150
    nearest_count 10
    error_bound 0.5
    recursion_limit 3
    low_error_factor 0.5
    gray_threshold 0.0
    minimum_reuse 0.005
    maximum_reuse 0.2
    brightness 1
    adc_bailout 0.005
  }
}
camera{ 
  location <14.6*cos(clock),14.6*sin(clock),10>
  look_at 0
  angle 7
  up <0,0,1>
  right <1,0,0>
}
//Try keep some light.
light_source{ <-1000*sin(clock/1.5), 0, 1000*cos(clock/1.5)> White }
#declare cloud_rnd_x = RANDOM_1;
#declare cloud_rnd_y = RANDOM_2;
#declare cloud_rnd_z = RANDOM_3;
#declare wrink_rnd_x = RANDOM_4;
#declare wrink_rnd_y = RANDOM_5;
#declare wrink_rnd_z = RANDOM_6;
#declare agate_rnd_x = RANDOM_7;
#declare agate_rnd_y = RANDOM_8;
#declare agate_rnd_z = RANDOM_9;

#declare base_rnd_x = RANDOM_10;
#declare base_rnd_y = RANDOM_11;
#declare base_rnd_z = RANDOM_12;

#declare cloud_angle_rnd_x = RANDOM_13;
#declare cloud_angle_rnd_y = RANDOM_14;
#declare cloud_angle_rnd_z = RANDOM_15;

// wrink_rnd_x,wrink_rnd_y,wrink_rnd_z, 
// base_rnd_x,base_rnd_y,base_rnd_z,
//

#declare ice_t = 0.6; //The below not changable as-is.
#declare best_temp = 0.8;
#declare temp_range = 0.2;

#declare cloud_h = 0.3*ATM_H;
#declare air_density = 4;

#declare water_eff_h = 0.03;

#declare water_g = 0;
#declare water_b = 1;

#declare rotate_angle = 2*clock*180/pi;

//Might attempt adding  mountains?
#declare base_hfun = function (x,y,z)
{ f_noise3d(x+base_rnd_x,y+base_rnd_y,z+base_rnd_z) } 

#declare wrinkles_tmplt= function{ pigment{ wrinkles 
  scale 1 
  translate <wrink_rnd_x,wrink_rnd_y,wrink_rnd_z>} }
#declare wrinkles_fun = function(x,y,z)
{ wrinkles_tmplt(x,y,z).red }

#declare hfun = function (x,y,z) {
  max((base_hfun(x,y,z) + 10*(base_hfun(x,y,z)-WATER_H)*wrinkles_fun(x,y,z))/
      (1+10*(1-WATER_H)),0) }

#declare normal_fun = function(x,y,z) { max(WATER_H,hfun(x,y,z)) }

#declare light_inpr = function (y){ SUN_HEAT*sqrt(1-pow(y*PLANET_SCALE,2)) }

#declare rock_bottom = texture //Rock bottom
  { pigment
    { function{ hfun(x,y,z) }
      color_map 
      { [0   color rgb <.6, .4, .1>] //Sand
	[0.4 color rgb <.55, .25, .05>]
	[0.8 color rgb <0.4,0.4,0.4>] //Rock 
	[1   color rgb <0.4,0.4,0.4>] //Rock 
      }
    }
    normal{ function{ normal_fun(x,y,z) } }
    scale PLANET_SCALE
  }

//Lower regions have more moderate temperatures, weighted average with 
// the north and south.
#declare temperature = function (y,h)
{ (light_inpr(y) + 2*(light_inpr(abs(y)-0.1) + light_inpr(min(abs(y)+0.2,1)))
                    /(1+pow((h-WATER_H)/0.1,2)))/
  (1+ 4/(1+pow((h-WATER_H)/0.1,2)))*(1-pow(h,5)*DEFAULT_ATM_H/ATM_H)
}

#declare growth_level = function(temp,h)
{ max(0, 1-2*pow(h-WATER_H,2)*DEFAULT_ATM_H/ATM_H)/
  (1 + pow((temp - best_temp)/temp_range,2)) }

#declare growth = texture
  { pigment
    { function
      { min(max(0,growth_level(temperature(y,hfun(x,y,z)), hfun(x,y,z))),1) }
      color_map
      { [0.0,0.5 color rgbft <0,0,0,0,1> color rgbft <.07, .4, .3, 0,1>]
	[0.7 color rgb <.07, .4, .3>]
	[1 color rgb <.07, .4, .3>]
//	[0.95 1 color rgb <.1,0.4,0.2> color rgb <1,0.4,0.3>]
      }
    }
    normal{ function{ normal_fun(x,y,z) } }
    scale PLANET_SCALE
  }
#declare tropic_level = function(temp,h)
{ max(0, 1-4*pow(h-WATER_H,2)*DEFAULT_ATM_H/ATM_H)/
  (1 + 2*pow((temp - 1.1*best_temp)/temp_range,2)) }

#declare tropic = texture
  { pigment
    { function
      { min(max(0,tropic_level(temperature(y,hfun(x,y,z)), hfun(x,y,z))),1) }
      color_map
      { [0.0,0.5 color rgbft <0,0,0,0,1> color rgbft <.0,.3,0, 0,1>]
	[0.95 color rgb <.0, .3, 0>]
	[1 color rgb <.0, .3, 0>]
      }
    }
    normal{ function{ normal_fun(x,y,z) } }
    scale PLANET_SCALE
  }

#declare ocean_area = texture { 
  pigment {
    function{ hfun(x,y,z) }
    color_map {
      [0.00, WATER_H color rgb <0,water_g,water_b> //Water
	             color rgb <0,water_g,water_b>]
      [WATER_H+0.05, 1.0 color rgbft <0,water_g,water_b, 0,1> //No water
                         color rgbft <0,water_g,water_b, 0,1>]
    }
  }
  normal{ function{ normal_fun(x,y,z) } }
  scale PLANET_SCALE
}

#declare ice_area = texture
  { pigment
    { function{ min(max(0,temperature(y,hfun(x,y,z))),1) }
      color_map
      { [0.0 ice_t color rgb <0.9,0.9,1> color rgb <0.9,0.9,1>]
	[ice_t 1 color rgbft <0.9,0.9,1,0,1> color rgbft <0.9,0.9,1,0,1>]
      }
    }
    normal
    { function{ normal_fun(x,y,z)+0.05 } }
    scale PLANET_SCALE
  }

#declare surf_dist = function(x,y,z)
{ sqrt(x*x+y*y+z*z)-1 }

#declare air = media {
  absorption rgb<0,0.5,0.6> //Lower these
  scattering{1, rgb<0.3,0.3,0.6>}
  density {
    function{ air_density*exp(-surf_dist(x,y,z)/(0.3*ATM_H))}
    color_map {
      [0.0 color rgb <0,0,0>]
      [1.0 color rgb <0.6,0.6,1>] } }} //less R

#declare agate_tmplt = function
  { pigment{ agate
             color_map { [0.0 color rgb <0, 0, 0>]
	                 [1.0 color rgb <1, 1, 1>] }
    translate <agate_rnd_x,agate_rnd_y,agate_rnd_z>
  } }

#declare agate_fun = function(x,y,z)
  { agate_tmplt(x,y,z).red }
  
#declare angle_time = function(x,y,z)
  { f_noise3d(2*x + cloud_angle_rnd_x,
              2*(y+ 2*cos(clock)) + cloud_angle_rnd_y,
              2*(z+ 2*sin(clock))  + cloud_angle_rnd_z) }
  
//This 'flows' planet-scale i want 'dynamics'
#declare cloud_comb = function(x,y,z) 
  { agate_fun(x, y + 2*sin(angle_time(x,y,z)), z + 2*cos(angle_time(x,y,z)))
    *f_noise3d(x,y +sin(angle_time(x,y,z)),z +cos(angle_time(x,y,z)))
  }
  
//Thanks to http://geomorph.sourceforge.net
#declare CLOUDS = 
  pigment {
    function{ cloud_comb(x,y,z) }
    translate <cloud_rnd_x,cloud_rnd_y,cloud_rnd_z>
    warp{ turbulence 0.2+0.05*cos(clock) }
    color_map {
      [0.2 - 0.2*CLOUDINESS rgbt <1,1,1,1>] 
      [0.95- 0.2*CLOUDINESS rgbt <0.9,0.9,0.9,0.1>] 
      [1.0                  rgbt <0.8,0.8,0.8,0.2>]
    }
  }
  
#declare SMALLER_CLOUDS =
  pigment {
    function{ cloud_comb(x,y,z) }
    translate <cloud_rnd_x,cloud_rnd_y,cloud_rnd_z>
    warp{ turbulence 0.2+0.05*cos(clock) }
    color_map {
      [0.4 - 0.2*CLOUDINESS rgbt <1,1,1,1>] 
      [0.95- 0.2*CLOUDINESS rgbt <1,1,1,0.2>]
      [1.0                  rgbt <0.9,0.9,0.9,0.2>]
    }
  }
sphere { <0,0,0>, 1
 texture{ rock_bottom }
texture{ growth }
texture{ tropic }
texture{ ocean_area }
texture{ ice_area }

}
sphere{ <0,0,0> 1
  hollow 
  pigment{ Clear }
  interior{ media{air} }
  scale 1+ATM_H
}

sphere //Clouds(1)
  { <0,0,0>, 1+cloud_h
    //Temporary in order to develop the terrain,
    //pigment{color red 0.8 green 0.8 blue 1  transmit 0.9 }
    pigment{ CLOUDS }
    rotate 2*clock*180/pi
  }
    
sphere //Clouds(2)
  { <0,0,0>, 1+1.2*cloud_h
    pigment{ SMALLER_CLOUDS }
    rotate 2*clock*180/pi
  }
sphere //Clouds(2)
  { <0,0,0>, 1+0.8*cloud_h
    pigment{ SMALLER_CLOUDS }
    rotate 2*clock*180/pi
  }
