#version USE_POVRAY_VERSION;
// Copyright (c) 2013 Jasper den Ouden, under the MIT license, 
// see ../doc/mit.txt from the project directory.

#include "colors.inc"
#include "functions.inc"

#declare atm_h = ATM_H;
#declare cloud_h = CLOUD_H;

#declare cloudiness = 0;

#declare water_h = 0;

#declare sun_heat = SUN_HEAT;
#declare planet_scale = 0.2;
#declare default_atm_h=0.03;
#declare ice_t = 0.6; 

global_settings
{ assumed_gamma 1.0
  radiosity {
    pretrace_start 0.08
    pretrace_end   0.01
    count 150
    nearest_count 10
    error_bound 0.5
    recursion_limit 3
    low_error_factor 0.5
    gray_threshold 0.0
    minimum_reuse 0.005
    maximum_reuse 0.2
    brightness 1
    adc_bailout 0.005
  }
}

#declare view_angle=VIEW_ANGLE;
camera{
  location <18*cos(view_angle),0,18*sin(view_angle)>
  look_at 0
  angle 7
  up <0,0,1>
  right <1,0,0>
}
light_source{ <1000*sin(clock), 0, 1000*cos(clock)> White }

#declare hfun_tmplt = function{ pigment {
  image_map {
    png HFUN_PNG //"hm.bmp.png"
    map_type 0
    interpolate 0
  }}}
#declare hfun = function(x,y,z){ hfun_tmplt(x,y,z).red }

#declare light_inpr = function (y){ sun_heat*sqrt(1-pow(y*planet_scale,2)) }

//Lower regions have more moderate temperatures, weighted average with 
// the north and south.
#declare temperature = function (y,h){
  (light_inpr(y) + 2*(light_inpr(abs(y)-0.1) + light_inpr(min(abs(y)+0.2,1)))
                    /(1+pow((h-water_h)/0.1,2)))/
  (1+ 4/(1+pow((h-water_h)/0.1,2)))*(1-pow(h,5)*default_atm_h/atm_h)
}

#declare ice_area = texture
  { pigment
    { function{ min(1,max(0,temperature(y,hfun(x,y,z)))) }
      color_map
      { [0.0 ice_t color rgb <0.9,0.9,1>     color rgb <0.9,0.9,1>]
	[ice_t 1   color rgbft <0.9,0.9,1,0,1> color rgbft <0.9,0.9,1,0,1>]
      }
    }
    normal
    { function{ hfun(x,y,z) } }
    scale planet_scale
    finish
    { diffuse 0.7
      specular 0.0
      roughness 1.0
    }
  }

sphere { <0,0,0>, 1
  texture {
    uv_mapping
    pigment{ image_map{
	png UV_PNG //"tex.bmp.png"
	map_type 0
	interpolate 0
    }}
    normal{ function{ hfun(x,y,z) } }
  } 
//Was hoping to add ice caps automatically, but 
// Parse Error: Cannot layer over a patterned texture.
//(that is not a parse error, btw.)
  OTHER_TEXTURE
}

#declare surf_dist = function(x,y,z)
{ sqrt(x*x+y*y+z*z)-1 }
#declare air =
      media {
	absorption rgb<0,0.5,0.6> //Lower these
	scattering{1, rgb<0.3,0.3,0.8>}
	density {
	  function{exp(-surf_dist(x,y,z)/(3*atm_h))}
	  color_map {
	    [0.0 color rgb <0,0,0>]
	    [1.0 color rgb <1,1,1>] } }} //less R

//Air!
sphere{ <0,0,0> 1
  hollow 
  pigment{ Clear }
  interior{ media{air} }
  scale 1+atm_h
}

#declare GRANITE = function
  { pigment{ granite
             color_map { [0.0 color rgb <0, 0, 0>]
	                 [1.0 color rgb <1, 1, 1>] }} }
    
#declare angle_time = function(x,y,z)
  { f_noise3d(2*x, 
              CLOUD_ANGLE_RND + 2*(y+ 2*cos(clock)),
              2*(z+ 2*sin(clock))) }

//This 'flows' planet-scale i want 'dynamics'
#declare cloud_comb = function(x,y,z) 
  { GRANITE(x+CLOUD_RND, y + 2*sin(angle_time(x,y,z)), 
            z + 2*cos(angle_time(x,y,z))).red*
           pow(f_noise3d(x,y,z),CLOUD_POW) }
  
//Thanks to http://geomorph.sourceforge.net
#declare CLOUDS = 
  pigment {
    function{ cloud_comb(x,y,z) }
    warp{ turbulence 0.2+0.05*cos(clock) }
    color_map {
      [0.2 - 0.2*cloudiness rgbt <1,1,1,1>] 
      [0.95- 0.2*cloudiness rgbt <0.9,0.9,0.9,0.1>] 
      [1.0                  rgbt <0.8,0.8,0.8,0.2>]
    }
  }
  
#declare SMALLER_CLOUDS =
  pigment {
    function{ cloud_comb(x,y,z) }
    warp{ turbulence 0.2+0.05*cos(clock) }
    color_map {
      [0.4 - 0.2*cloudiness rgbt <1,1,1,1>] 
      [0.95- 0.2*cloudiness rgbt <1,1,1,0.2>]
      [1.0                  rgbt <0.9,0.9,0.9,0.2>]
    }
  }

sphere //Clouds(1)
  { <0,0,0>, 1+atm_h*0.2
    //Temporary in order to develop the terrain,
    //pigment{color red 0.8 green 0.8 blue 1  transmit 0.9 }
    pigment{ CLOUDS }
  }
  
  
sphere //Clouds(2)
  { <0,0,0>, 1+atm_h*0.3
    pigment{ SMALLER_CLOUDS }
  }
 